require 'active_record'

include ActiveRecord::Tasks

class SeedLoader
  def initialize(seed_file)
    @seed_file = seed_file
  end

  def load_seed
    raise "Seed file '#{@seed_file}' does not exist" unless File.file?(@seed_file)
    load @seed_file
  end
end

root = File.expand_path '../../..', __FILE__

DatabaseTasks.tap do |t|
  Application.start(:mysql)

  t.env = Application.env.to_s
  t.database_configuration = Application['mysql.database_configuration']
  t.db_dir = File.join(root, 'db')
  t.fixtures_path = File.join(root, 'test', 'fixtures')
  t.migrations_paths = [File.join(root, 'db','migrate')]
  t.seed_loader = SeedLoader.new(File.join(root, 'db', 'seeds.rb'))
  t.root = root
end

# For reference to db rake tasks:
# https://github.com/rails/rails/blob/master/activerecord/lib/active_record/railties/databases.rake
load "active_record/railties/databases.rake"

namespace :db do
  namespace :schema do
    desc "invoke db:schema:load if schema doesn't exist"
    task :load_if_new do
      logger = Application['logger']
      if ActiveRecord::Base.connection.tables.include?("schema_migrations")
        logger.info "Skipping schema load. it already exists"
      else
        logger.info "loading initial schema"
        Rake::Task["db:schema:load"].invoke
      end
    end
  end
end

# Used to create Migration files, usage:
# bundle exec rake 'g:migration[create_account_api_access_statuses]'
namespace :g do
  desc "Generate migration"
  task :migration, [:file_name] do |t, args|
    name = args[:file_name] || raise("Specify name: rake 'g:migration[your_migration]'")
    timestamp = Time.now.strftime("%Y%m%d%H%M%S")
    path = File.join(root, 'db', 'migrate', "#{timestamp}_#{name}.rb")
    migration_class = name.split("_").map(&:capitalize).join

    File.open(path, 'w') do |file|
      file.write <<-EOF
class #{migration_class} < ActiveRecord::Migration[6.0]
  def change
  end
end
      EOF
    end

    puts "Migration #{path} created"
    abort # needed stop other tasks
  end
end
